-- This file is part of Hypha.
--
-- Hypha is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- Hypha is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with Hypha.  If not, see <http://www.gnu.org/licenses/>.
--
-- Copyright 2013 Luca Padovani

{
module Lexer (Token(..), alexScanTokens) where
}

%wrapper "basic"

$digit = 0-9            -- digits
$alpha = [a-zA-Z]       -- alphabetic characters

tokens :-

  $white+                               ;
  "--".*                                ;
  $digit+                               { \s -> INT (read s) }
  "!"                                   { \_ -> EMARK }
  "?"                                   { \_ -> QMARK }
  "+"					{ \_ -> PLUS }
  "-"					{ \_ -> MINUS }
  "*"					{ \_ -> STAR }
  "/"					{ \_ -> SLASH }
  "|"                                   { \_ -> PAR }
  "("                                   { \_ -> LPAR }
  ")"                                   { \_ -> RPAR }
  "="                                   { \_ -> Lexer.EQ }
  "<"                                   { \_ -> Lexer.LT }
  ">"                                   { \_ -> Lexer.GT }
  "<="                                  { \_ -> Lexer.LE }
  ">="                                  { \_ -> Lexer.GE }
  "<>"                                  { \_ -> Lexer.NE }
  "["                                   { \_ -> LBRACK }
  "]"                                   { \_ -> RBRACK }
  "{"                                   { \_ -> LBRACE }
  "}"                                   { \_ -> RBRACE }
  ","                                   { \_ -> COMMA }
  "."                                   { \_ -> DOT }
  ":"                                   { \_ -> COLON }
  ";"                                   { \_ -> SEMICOLON }
  "=>"                                  { \_ -> ARROW }
  "_"					{ \_ -> UNDERSCORE }
  $alpha [$alpha $digit \_ \']*         { \s -> lookupId s }
  
{


keywords :: [(String, Token)]
keywords = [("true", TRUE),
	    ("false", FALSE),
	    ("if", IF),
	    ("then", THEN),
	    ("else", ELSE),
	    ("let", LET),
	    ("in", IN),
	    ("inl", INL),
	    ("inr", INR),
	    ("fst", FST),
	    ("snd", SND),
	    ("new", NEW),
	    ("case", CASE),
	    ("of", OF),
	    ("and", AND),
	    ("or", OR),
	    ("not", NOT)]

lookupId :: String -> Token
lookupId s = case lookup s keywords of
	       Nothing -> ID s
	       Just tok -> tok

-- Each right-hand side has type :: String -> Token

-- The token type:
data Token =
    INT Int     |
    TRUE	|
    FALSE	|
    EMARK       |
    QMARK       |
    STAR        |
    IF          |
    LET         |
    CASE        |
    OF          |
    THEN        |
    ELSE        |    
    PAR         |
    NEW         |
    LPAR        |
    RPAR        |
    EQ          |
    IN          |
    INL         |
    INR         |
    FST		|
    SND		|
    GT          |
    LT          |
    GE          |
    LE          |
    NE          |
    AND		|
    OR		|
    NOT		|
    LBRACK      |
    RBRACK      |
    LBRACE      |
    RBRACE      |
    PLUS	|
    MINUS	|
    SLASH	|
    COMMA       |
    DOT         |
    COLON       |
    SEMICOLON	|
    ARROW       |
    UNDERSCORE	|
    ID String   |
    ERR
    deriving (Eq,Show)
}
